<?php
/**
 * WeCodeArt Framework.
 *
 * WARNING: This file is part of the core WeCodeArt Framework. DO NOT edit this file under any circumstances.
 * Please do all modifications in the form of a child theme.
 *
 * @package 	WeCodeArt Framework
 * @subpackage 	Support\FileSystem
 * @copyright   Copyright (c) 2023, WeCodeArt Framework
 * @since 		5.0.0
 * @version		5.3.3
 */

namespace WeCodeArt\Support;

defined( 'ABSPATH' ) || exit;

use WeCodeArt\Singleton;
use WeCodeArt\Integration;
use WeCodeArt\Conditional\Traits\No_Conditionals;

/**
 * Manages the Theme Uploads.
 */
class FileSystem implements Integration {

	use Singleton;
	use No_Conditionals;

	/**
	 * Current folder
	 *
	 * @since 	5.0.0
	 * @var 	string
	 */
	public	$folder = '';

	/**
	 * Send to contructor.
	 */
	public function init() {
		$this->set_folder();
	}

	/**
	 * Register hooks
	 */
	public function register_hooks() {}

	/**
	 * Function to set folder in WordPress Filesystem.
	 *
	 * @since   5.0.0
	 *
	 * @return  bool
	 */
	public function set_folder( $folder = '' ) {
		$theme_dir = wecodeart_config( 'paths' )['directory'];

		if ( ! is_writable( $theme_dir ) ) {
			return false;
		}

		$this->folder = implode( DIRECTORY_SEPARATOR, array_filter( [ $theme_dir, 'assets', $folder ] ) );

		wp_mkdir_p( wp_normalize_path( $this->folder ) );

		return $this;
	}

	/**
	 * Function to delete folder from WordPress Filesystem.
	 *
	 * @since   5.0.6
	 *
	 * @return  boolean
	 */
	public function clean_folder( $path = '' ) {
		$folder = wp_normalize_path( implode( DIRECTORY_SEPARATOR, array_filter( [ $this->folder, $path ] ) ) );
		
		if( is_dir( $folder ) ) {
			$objects = scandir( $folder );

			foreach ( $objects as $object ) { 
				if ( $object !== '.' && $object !== '..' ) {
					$path = implode( DIRECTORY_SEPARATOR, [ $folder, $object ] );
					
					if ( is_dir( $path ) && ! is_link( $path ) ) rmdir( $path );
					else unlink( $path ); 
				}
			}

			return rmdir( $folder );
		}

		return true;
	}
	
	/**
	 * Function to create file from WordPress Filesystem.
	 *
	 * @since   5.0.0
	 *
	 * @return  bool
	 */
	public function create_file( $filename, $content ) {
		// Delete old file
		$this->delete_file( $filename );

		// Create new file
		return file_put_contents( $this->get_file_url( $filename ), $content );
	}

	/**
	 * Function to delete file from WordPress Filesystem.
	 *
	 * @since   5.0.0
	 *
	 * @return  bool
	 */
	public function delete_file( $filename ) {
		// If no specific file, bail
		if ( $this->has_file( $filename ) ) {
			return unlink( $this->get_file_url( $filename ) );
		}
	}

	/**
	 * Check if we have a file
	 *
	 * @since   5.0.0
	 *
	 * @return  bool
	 */
	public function has_file( $file ) {
		return is_file( $this->get_file_url( $file ) );
	}

	/**
	 * Get File url.
	 *
	 * @return  string File url.
	 */
	public function get_file_url( $filename, $link = false ) {
		$file = implode( DIRECTORY_SEPARATOR, array_filter( [ $this->folder, $filename ] ) );
		
		if( $link ) {
			$paths = wecodeart_config( 'paths' );

			$file = wp_normalize_path( str_replace( $paths['directory'], $paths['uri'], $file ) );
		}

		return $file;
	}
}